@@ -1,6 +1,6 @@
 .clean <- function(LHS, RHS) {
 
-  idx <- which(colSums(RHS) == 0)
+  idx <- which(Matrix::colSums(RHS) == 0)
 
   if (length(idx) > 0) {
 
