@@ -65,10 +65,10 @@ FormalContext <- R6::R6Class(
     expanded_grades_set = NULL,
 
     #' @field concepts The concept lattice associated to the formal context as a \code{\link{ConceptLattice}}.
-    concepts = NULL,
+    concepts = "Not computed yet",
 
     #' @field implications A set of implications on the formal context as an \code{\link{ImplicationSet}}.
-    implications = NULL,
+    implications = "Not computed yet",
 
     #' @description
     #' Creator for the Formal Context class
@@ -244,6 +244,7 @@ FormalContext <- R6::R6Class(
     #'
     #' @param attributes  The attributes to scale
     #' @param type        Type of scaling.
+    #'
     #' @param ...
     #'
     #' @details
@@ -251,6 +252,8 @@ FormalContext <- R6::R6Class(
     #' so that \code{scalingRegistry$get_entries()} returns
     #' all types.
     #'
+    #' In the dots argument, the user can supply the value for \code{bg} (logical), which, if set to \code{TRUE}, indicates to compute background knowledge as implications on the scales; if \code{FALSE}, no implications will be computed on the scales.
+    #'
     #' @return The scaled formal context
     #' @export
     #' @examples
@@ -263,29 +266,43 @@ FormalContext <- R6::R6Class(
 
       # TODO: Check that the attributes are in self$attributes
 
+      bg <- FALSE
+      dots <- list(...)
+      if ("bg" %in% names(dots)) {
+
+        bg <- dots$bg
+        dots$bg <- NULL
+
+      }
+
       I <- self$incidence()
       for (att in attributes) {
 
         scaled <-
           scale_context(I, column = att,
                         type = type,
-                        ...)
+                        bg = bg,
+                        dots)
 
         I <- scaled$derived
         private$scales <- c(private$scales,
                             scaled$scale)
         names(private$scales)[length(private$scales)] <- att
 
-        # Add implications to the bg_implications
-        private$bg_implications <- combine_implications(
-          private$bg_implications,
-          scaled$bg_implications)
+        if (bg) {
 
-        # if (scaled$bg_implications$cardinality() > 0) {
-        #
-        #   private$bg_implications$to_basis()
-        #
-        # }
+          # Add implications to the bg_implications
+          private$bg_implications <- combine_implications(
+            private$bg_implications,
+            scaled$bg_implications)
+
+          # if (scaled$bg_implications$cardinality() > 0) {
+          #
+          #   private$bg_implications$to_basis()
+          #
+          # }
+
+        }
 
       }
 
@@ -1565,18 +1582,6 @@ FormalContext <- R6::R6Class(
 
       }
 
-      if (length(objects) > length(final_objects)) {
-
-        warning("An object you provided was not found in this context.", call. = FALSE, immediate. = TRUE)
-
-      }
-
-      if (length(attributes) > length(final_attributes)) {
-
-        warning("An attribute you provided was not found in this context.", call. = FALSE, immediate. = TRUE)
-
-      }
-
       if (length(objects) == 0) objects <- self$objects
       if (length(attributes) == 0) attributes <- self$attributes
 
